import { __decorate } from "tslib";
import { readdir } from "fs";
import { resolve, sep } from "path";
import { BindOption, Converter, DeclarationReflection, ReflectionCategory, ReflectionGroup, ReflectionKind, Renderer, } from "typedoc";
const isClientModel = (model) => model?.sources?.[0]?.fullFileName.includes(`${sep}clients${sep}`);
export class SdkIndexLinkClientPlugin {
    constructor(options, logger, renderer) {
        this.options = options;
        this.logger = logger;
        this.renderer = renderer;
        this.onConverterEnd = async (context) => {
            const project = context.project;
            if (!project.groups) {
                project.groups = [];
            }
            let group = project.groups.find((value) => value.title === this.defaultGroup);
            if (!group) {
                group = new ReflectionGroup(this.defaultGroup);
                project.groups.push(group);
            }
            const modules = project.getChildrenByKind(ReflectionKind.SomeModule);
            await this.registerClients(project, modules);
            group.categories = this.defineCategories(group, modules);
        };
        this.onPageBegin = (page) => {
            if (page.model.flags.includes("SkipRendering")) {
                page.preventDefault();
            }
            if (page.model.isProject()) {
                const group = page.model.groups.find((value) => value.title === this.defaultGroup);
                const clientsCategory = group.categories.find((value) => value.title === "Clients");
                for (const child of clientsCategory.children || []) {
                    child.url = `https://docs.aws.amazon.com/AWSJavaScriptSDK/v3/latest/clients/${child.originalName}/`;
                    child.name = `@aws-sdk/${child.originalName.replace("client-", "")}`;
                }
            }
        };
        this.renderer.application.converter.on(Converter.EVENT_END, this.onConverterEnd);
        this.renderer.on(Renderer.EVENT_BEGIN_PAGE, this.onPageBegin);
    }
    async registerClients(project, modules) {
        return new Promise((resolvePromise, rejectPromise) => {
            const clientPath = resolve("./clientDocs/clients");
            readdir(clientPath, (err, clientFiles) => {
                if (err)
                    rejectPromise(err);
                clientFiles.forEach((clientName) => {
                    if (clientName.includes("client") && !clientName.includes("documentation")) {
                        const clientDeclaration = new DeclarationReflection(clientName, ReflectionKind.Module, project);
                        project.registerReflection(clientDeclaration);
                        modules.push(clientDeclaration);
                    }
                });
                resolvePromise();
            });
        });
    }
    defineCategories(group, reflections) {
        const categories = group.categories || [];
        if (this.isCategorized(categories))
            return group.categories;
        const clients = new ReflectionCategory("Clients");
        const packages = new ReflectionCategory("Packages");
        const libs = new ReflectionCategory("Libraries");
        reflections.forEach((reflection) => {
            if (this.isClient(reflection)) {
                clients.children.push(reflection);
                reflection.flags.includes("SkipRendering");
            }
            else if (this.isLib(reflection)) {
                libs.children.push(reflection);
            }
            else {
                packages.children.push(reflection);
            }
        });
        categories.push(...[clients, libs, packages]);
        return categories;
    }
    isCategorized(categories) {
        const childrenNames = categories.map((child) => child.title);
        return (childrenNames.includes("Clients") && childrenNames.includes("Packages") && childrenNames.includes("Libraries"));
    }
    isClient(item) {
        return isClientModel(item) || item.name.includes("client-");
    }
    isLib(item) {
        return item?.sources?.[0].fileName.startsWith(`lib${sep}`);
    }
}
__decorate([
    BindOption("out")
], SdkIndexLinkClientPlugin.prototype, "outputDirectory", void 0);
__decorate([
    BindOption("clientDocs")
], SdkIndexLinkClientPlugin.prototype, "clientDocs", void 0);
__decorate([
    BindOption("defaultGroup")
], SdkIndexLinkClientPlugin.prototype, "defaultGroup", void 0);
